package mimuw.mmf.clusterers.hierarchical;

import java.text.DecimalFormat;
import java.text.DecimalFormatSymbols;
import java.util.Formatter;
import java.util.LinkedList;
import java.util.List;

import mimuw.mmf.clusterers.Cluster;

public class HierarchicalCluster<T> implements Cluster<T>{
	
	private final HierarchicalCluster<T> first;
	private final HierarchicalCluster<T> second;
	private final double distance;
	
	public HierarchicalCluster(HierarchicalCluster<T> first, HierarchicalCluster<T> second, double distance) {
		super();
		this.first = first;
		this.second = second;
		this.distance = distance;
	}

	public int size(){
		return first.size() + second.size();
	}
	
	public HierarchicalCluster<T> first(){
		return first;
	}
	
	public HierarchicalCluster<T> second(){
		return second;
	}

	public double distance() {
		return distance;
	}
	
	@Override
	public String toString(){
		return '('+ first.toString() + ',' + second.toString() + ")";
	}
	
	public String valsToString(){
		return first.valsToString() + ", " + second.valsToString();
	}
	
	public List<T> getObjects(){
		List<T> objects = new LinkedList<T>();
		objects.addAll(first.getObjects());
		objects.addAll(second.getObjects());
		return objects;
	}
	
	public String fullNewickFormat(){
		StringBuffer buff = new StringBuffer();
		DecimalFormatSymbols dfs = new DecimalFormatSymbols();
		dfs.setDecimalSeparator('.');
		DecimalFormat df = new DecimalFormat("0.000", dfs);
		Formatter formatter = new Formatter(buff);
		formatter.format("(%s:%s,%s:%s)", first.fullNewickFormat(), df.format(distance), second.fullNewickFormat(), df.format(distance));
		return buff.toString();
	}
}
