package mimuw.mmf.finders;

import java.io.File;
import java.util.List;
import java.util.Map;

import mimuw.mmf.WrongParameterException;
import mimuw.mmf.motif.Motif;

import org.biojavax.bio.seq.RichSequence;

public interface MotifFinder {

	//public static final int DEFAULT_MOTIF_LENGTH = 8;
	public static final int MIN_MOTIF_LEN = 4;
	public static final int MAX_MOTIF_LEN = 50;
	//public static final boolean DEFAULT_BOTH_STRANDS = true;
	public static final int MIN_REPORTED_MOTIFS_NUMBER = 1;
	public static final int MAX_REPORTED_MOTIFS_NUMBER = 100;
	//public static final int DEFAULT_REPORTED_MOTIFS_NUMBER = 3;
	
	
	/**
	 * Returns the name of the section concerning this finder's section in the parameter file
	 */
	public String getProgramIdentifier();



	/**
	 * Sets all the parameters for motif-finding program as one string. The string must contain 
	 * all required parameters (except from the parameters set in code), and may contain 
	 * additional ones. For some programs (Weeder) the order of the parameters matters.
	 */
	public void setParametersAsSingleString(String pS) throws WrongParameterException;



	/**
	 * Runs the corresponding motif-finding program on a given FASTA file (already
	 * analysed, sequences from the file are read into the Map)
	 *  
	 * @param seqFile
	 * @param seqMap
	 * @return
	 * @throws MMFException
	 */
	public List<Motif> findMotifs (File seqFile, Map<String, RichSequence> seqMap) throws MMFException;


	
	public void setMotifLength(int len) throws WrongParameterException;
	
	public void setSearchBothStrands(boolean both) throws WrongParameterException;
	
	public void setDesiredNumberOfMotifs(int n) throws WrongParameterException;

}
