package mimuw.mmf.finders.bioprospector;


import static org.junit.Assert.assertEquals;
import mimuw.mmf.WrongParameterException;

import org.junit.After;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;

public class BioProspectorFinderTest {

	BioProspectorFinder finder;
	
	@BeforeClass
	public static void setUpBeforeClass() throws Exception {
	}

	@Before
	public void setUp() throws Exception {
		finder = new BioProspectorFinder();
	}
	
	@After
    public void tearDown() {
        finder.clean();
    }
	
	@Test
	public void checkIdentifier(){
		assertEquals("bioprospector", finder.getProgramIdentifier());
	}

	@Test
	public void testCorrectParameters() throws WrongParameterException{
		finder.setParametersAsSingleString("   -W 5 -n 7  -p 1 -a 0  -m 5.134 -d 1 -G 32");
	}

	@Test(expected=WrongParameterException.class)
	public void testNoSpaceParameters() throws WrongParameterException{
		finder.setParametersAsSingleString("   -a0  ");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testWrongParametern() throws WrongParameterException{
		finder.setParametersAsSingleString("-n 201");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testWrongParametere() throws WrongParameterException{
		finder.setParametersAsSingleString("-e 0");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testWrongParameterG() throws WrongParameterException{
		finder.setParametersAsSingleString("-G 51");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testWrongParameterd() throws WrongParameterException{
		finder.setParametersAsSingleString("-d 3");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testWrongParameterr() throws WrongParameterException{
		finder.setParametersAsSingleString("-r abc");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testProhibitedParameterw() throws WrongParameterException{
		finder.setParametersAsSingleString("-w 13");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testProhibitedParametero() throws WrongParameterException{
		finder.setParametersAsSingleString("-o abc.txt");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testProhibitedParameterf() throws WrongParameterException{
		finder.setParametersAsSingleString("-f file.zzz");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testProhibitedParameterb() throws WrongParameterException{
		finder.setParametersAsSingleString("-b /tmp/void.txt");
	}
	
	@Test(expected=WrongParameterException.class)
	public void testProhibitedParametert() throws WrongParameterException{
		finder.setParametersAsSingleString("-t 1");
	}
}