package mimuw.mmf.finders.bioprospector;

import static org.junit.Assert.assertEquals;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import mimuw.mmf.finders.MMFException;
import mimuw.mmf.motif.Motif;

import org.biojava.bio.BioException;
import org.biojava.bio.seq.DNATools;
import org.biojava.bio.symbol.FiniteAlphabet;
import org.biojavax.bio.seq.RichSequence;
import org.junit.Test;

public class BioProspectorResultsParserTest {

	@Test
	public void testParseFile() throws IOException, MMFException, BioException {
		
		File fileUnderTest = File.createTempFile("bioprospector-test", ".txt\n");
		BufferedWriter out = new BufferedWriter(new FileWriter(fileUnderTest));
		
		
		Map<String, RichSequence> seqMap = new HashMap<String, RichSequence>();
		FiniteAlphabet dnaAlphabet = DNATools.getDNA();
		
		RichSequence seqA = RichSequence.Tools.createRichSequence("seqA", "CGGTGAACACCAAGTGTTCAC", dnaAlphabet);
		RichSequence seqB = RichSequence.Tools.createRichSequence("seqB", "GTGGTGCACACTGGGTGTGCAC", dnaAlphabet);
		RichSequence seqC = RichSequence.Tools.createRichSequence("seqC", "AATGGTGTCCACGTGTGTCCAC", dnaAlphabet);
		seqMap.put(seqA.getName(), seqA);
		seqMap.put(seqB.getName(), seqB);
		seqMap.put(seqC.getName(), seqC);
		
		
		out.write("The highest scoring 3 motifs are:\n");
		out.write("Motif #1: (GGGTGTCCAC/GTGGACACCC)\n");
		out.write("******************************\n");
		out.write("Width (10, 0); Gap [0, 0]; MotifScore 1.458; Segments 2\n");
		out.write("\n");
		out.write("Blk1    A     C     G     T      Con  rCon Deg  rDeg\n");
		out.write("1     0.17  0.20  0.33  0.30      G    C    K    M\n");
		out.write("2     0.17  0.06  0.59  0.17      G    C    G    C\n");
		out.write("3     0.04  0.06  0.85  0.04      G    C    G    C\n");
		out.write("\n");
		out.write(">seqA seg 1	r1	-1	CGGTGAACAC\n");
		out.write(">seqB seg 1	r2	-2	TGGTGCACAC\n");
		out.write("******************************\n");
		out.write("\n");
		out.write("Motif #2: (GGGTGTCCAC/GTGGACACCC)\n");
		out.write("******************************\n");
		out.write("Width (10, 0); Gap [0, 0]; MotifScore 1.458; Segments 2\n");
		out.write("\n");
		out.write("Blk1    A     C     G     T      Con  rCon Deg  rDeg\n"); 
		out.write("1     0.17  0.20  0.33  0.30      G    C    K    M\n");
		out.write("2     0.17  0.06  0.59  0.17      G    C    G    C\n");
		out.write("3     0.04  0.06  0.85  0.04      G    C    G    C\n");
		out.write("\n");
		out.write(">seqB seg 2	f1	-13	GGGTGTGCAC\n");
		out.write(">seqC seg 2	f12	-11	GTGTGTCCAC\n");
		out.write("******************************\n");
		out.write("\n");
		out.write("Motif #3: (GGGTGTCCAC/GTGGACACCC)\n");
		out.write("******************************\n");
		out.write("Width (10, 0); Gap [0, 0]; MotifScore 1.458; Segments 2\n");
		out.write("\n");
		out.write("Blk1    A     C     G     T      Con  rCon Deg  rDeg\n"); 
		out.write("1     0.17  0.20  0.33  0.30      G    C    K    M\n");
		out.write("2     0.17  0.06  0.59  0.17      G    C    G    C\n");
		out.write("3     0.04  0.06  0.85  0.04      G    C    G    C\n");
		out.write("\n");
		out.write(">seqA seg 1	r1	-1	CGGTGAACAC\n");
		out.write(">seqA seg 2	f1	-12	AAGTGTTCAC\n");
		out.write("******************************\n");
		out.write("\n");
		out.write("Total time 0:0:0.\n");
		out.close();
		
		BioProspectorResultsParser parser = new BioProspectorResultsParser(
				fileUnderTest.getAbsolutePath());
		
		List<Motif> motifs = parser.parseFile(seqMap);
		
		assertEquals(3, motifs.size());
		
		for (Motif m: motifs){
			assertEquals(10, m.length());
			assertEquals(2, m.getOccurrences().size());
		}
		if (!fileUnderTest.delete())
			throw new IOException("Failed to delete file: " + fileUnderTest.getAbsolutePath());
	}

}
