package mimuw.mmf.finders.mdscan;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.biojavax.bio.seq.RichSequence;

import mimuw.mmf.finders.AbstractResultsParser;
import mimuw.mmf.finders.MMFException;
import mimuw.mmf.motif.Motif;
import mimuw.mmf.motif.MotifOccurrence;
import mimuw.mmf.motif.OccurrenceBeyondSequenceException;
import mimuw.mmf.util.Logger;

public class MDscanResultsParser extends AbstractResultsParser {

	public MDscanResultsParser(String resultsFile) throws MMFException {
		super(resultsFile);
	}

	private MotifOccurrence parseMotifOccurrence(Map<String, RichSequence> seqMap,
			String line, int lineNo, int motifLen) throws OccurrenceBeyondSequenceException {

		MotifOccurrence result;
		String sequenceName;
		int pos = 0;
		boolean forward = true;

		String[] tokens = line.split("\\s+");
		sequenceName = tokens[0].substring(1);//TODO: not really: id may be longer than one cell...
		RichSequence seq = seqMap.get(sequenceName);
		if (seq == null)
			wrongFormat("some known sequence", sequenceName);
		int seqLen = seq.length();
		
		int lastIndex = tokens.length - 1;
		if (((lastIndex - 2) < 0) || (!"St".equals(tokens[lastIndex - 2])))
			wrongFormat();
		
		String location = tokens[lastIndex - 1];
		try {
			pos = new Integer(location.substring(1));
			switch (location.charAt(0)){
			case 'f': //forward
				break;
			case 'b': //reverse
				forward = false;
				break;
			default:
				wrongFormat();
			}
		} catch (NumberFormatException e){
			wrongFormat();
		}
		
		// in mdscan output position for reverse strand is counted from the end
		if (!forward)
			pos = seqLen + 2 - motifLen - pos;
		result = new MotifOccurrence(seq, pos, forward, motifLen);
		return result;
	}
	
	@Override
	protected Motif parseMotif(Map<String, RichSequence> seqMap) {
		
		Motif result = null;

		while (line != null && (!line.trim().startsWith("Mtf"))){
			readln();
		}
		if (line == null) return result;


		String[] tokens = line.split("\\s+");
		String motifNumber = tokens[1];
		int length = new Integer(tokens[2]);
		Logger.debug("Found motif: %s length: %d\n", motifNumber, length);
		
		while (readln() != null){
			if (line.trim().startsWith(">"))
				break;
		}
		if (line == null) 
			wrongFormat("no sequence section for motif " + motifNumber );
		
		List<MotifOccurrence> occurrences = new LinkedList<MotifOccurrence>();
		while (line != null){
			if (!line.startsWith(">"))
				break;
			else try { 
				MotifOccurrence o = parseMotifOccurrence(seqMap, line, count, length);
				occurrences.add(o);
			} catch (OccurrenceBeyondSequenceException e) {
				Logger.warn("WARNING: %s. \n  Occurrence removed.\n", e.getMessage());
			}
			readln();
		}
		result = new Motif(length, occurrences, MDscanFinder.IDENTIFIER, motifIndex);
		this.motifIndex ++;
		return result;
	}

}
