package mimuw.mmf.finders.meme;

import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import org.biojavax.bio.seq.RichSequence;

import mimuw.mmf.finders.AbstractResultsParser;
import mimuw.mmf.finders.MMFException;
import mimuw.mmf.motif.Motif;
import mimuw.mmf.motif.MotifOccurrence;
import mimuw.mmf.motif.OccurrenceBeyondSequenceException;
import mimuw.mmf.util.Logger;

public class MEMEResultsParser extends AbstractResultsParser {

	protected MEMEResultsParser(String resultsFile) throws MMFException{
		super(resultsFile); 
	}

	private MotifOccurrence parseMotifOccurrence(Map<String, RichSequence> seqMap, 
			String line, int lineNo, int motifLen) throws OccurrenceBeyondSequenceException {

		MotifOccurrence result;
		String sequenceName;
		RichSequence seq;
		int pos = 0;
		boolean forward = true;
		
		String[] tokens = line.split("\\s+");
		sequenceName = tokens[0];
		seq = seqMap.get(sequenceName);
		try {
			if (tokens[1].matches("[\\+\\-]")){	
				// option -revcomp => extra column 
				forward = tokens[1].matches("\\+");
				pos = new Integer(tokens[2]);
			} else {
				// no -revcomp
				pos = new Integer(tokens[1]);
			}
		} catch (NumberFormatException e){
			wrongFormat();
		}
		result = new MotifOccurrence(seq, pos, forward, motifLen);
		return result;
	}

	
	
	@Override
	protected Motif parseMotif(Map<String, RichSequence> seqMap) {

		Logger.trace(">> parseMotif()\n");
		Motif result = null;

		while (line != null && (!line.trim().startsWith("MOTIF"))){
			readln();
		}
		if (line == null) {
			Logger.trace("line was null - EOF\n"); //TODO - delete
			return result;
		}


		String[] tokens = line.split("\\s+");
		String motifNumber = tokens[1];
		int length = new Integer(tokens[4]);
		
		while (readln() != null){
			if (line.trim().startsWith("Sequence name"))
				break;
		}
		if (line == null) 
			wrongFormat("'Sequence name' section for motif " + motifNumber );
		readln();	// line with hyphens
		
		List<MotifOccurrence> occurrences = new LinkedList<MotifOccurrence>();
		
		while (readln() != null){
			if (line.startsWith("---"))
				break;
			else try { 
				MotifOccurrence o = parseMotifOccurrence(seqMap, line, count, length);
				occurrences.add(o);
			} catch (OccurrenceBeyondSequenceException e) {
				Logger.warn("WARNING: %s. \n  Occurrence removed.\n", e.getMessage());
			}
		}
		result = new Motif(length, occurrences, MEMEFinder.IDENTIFIER, motifIndex);
		this.motifIndex ++;
		Logger.trace("<< parseMotif()\n");
		return result;
	}
}
