package mimuw.mmf.jaspar;

import java.io.Serializable;
import java.util.Formatter;
import java.util.List;

import mimuw.mmf.motif.AbstractMotif;
import mimuw.mmf.util.Constants;
import mimuw.mmf.util.Logger;

import org.biojava.bio.BioException;
import org.biojava.bio.dist.IndexedCount;
import org.biojava.bio.seq.DNATools;
import org.biojava.bio.symbol.FiniteAlphabet;

public class JasparMotif extends AbstractMotif implements Serializable {

	private static final long serialVersionUID = -3356494756225920704L;
	public static final String IDENTIFIER = "jaspar";
	
	private final String id;
	private final String name;
	private final String motifsClass;
	private final String species;
	
	private final FiniteAlphabet alphabet;
	
	public JasparMotif(final String id, final String name, final String theClass, 
			String species, final List<IndexedCount> tmpCounts) throws BioException 
	{
		super(tmpCounts.toArray(new IndexedCount[tmpCounts.size()]), Constants.getBgDist());
		this.id = id;
		this.name = name;
		this.motifsClass = theClass;
		if (species == null)
			species = "";
		this.species = species;	
		alphabet = DNATools.getDNA();
		Logger.debug("JASPAR  %-16.16s   %-15.15s   %-15.15s    %.3f    %s\n", 
				getSpecies(), getMotifsClass(), getName(), avgInfo(), consensus());
	}

	public String getId() {
		return id;
	}
	public String getName() {
		return name;
	}
	public String getSpecies() {
		return species;
	}
	public String getMotifsClass() {
		return motifsClass;
	}
	
	@Deprecated
	public double getWeight(){
		return 0.7 * avgInfo();
	}
	
	@Override
	public String toString(){
		StringBuffer buff = new StringBuffer();
		Formatter formatter = new Formatter(buff);
		formatter.format("%s_%s[%s_%s]", consensus(), origin().substring(0, 2), name, motifsClass.replace(',', '+'));
		return buff.toString();
	}

	@Override
	public String origin() {
		return IDENTIFIER;
	}

	@Override
	public FiniteAlphabet getAlphabet() {
		return alphabet;
	}
}