package mimuw.mmf.util;

import java.util.List;

import mimuw.mmf.clusterers.DistanceCounter;

public class SimilarityFilter <T> {

	private final DistanceCounter<T> counter;
	private final double threshold;
	
	public SimilarityFilter(DistanceCounter<T> c, double threshold) {
		this.counter = c;
		this.threshold = threshold;
	}
	
	/**
	 * Compares every two objects from the list,  and removes the latter one if only 
	 * the distance between them is lower than the given threshold
	 * @param <E>
	 * @param toFilter
	 */
	public <E extends T>void filter(List<E> toFilter){
		
		for(int index1=toFilter.size()-1; index1 >= 0; index1--){
			T obj1 = toFilter.get(index1);
			for (int index2 = index1 - 1; index2 >= 0; index2--){
				T obj2 = toFilter.get(index2);
				if (counter.distance(obj1, obj2) < threshold){
					Logger.info(" FILTER: removing: %s\n", obj1); // TODO: delete it
					toFilter.remove(index1);
					break;
				}
			}
		}
	}

}
